/*
** Vinculo_Test.c
**
** Copyright  2010 Future Devices International Limited
**
** C Source file for Vinculum II test applications
** Main module
**
** Author: FTDI
** Project: Vinculum II / Vinculo
** Module: Vinculum II Applications
** Requires: VOS, IOMux, USBSlave, USBSlaveFT232, IOCTL, GPIO
** Comments:
**
** History:
**  1  Initial version
**
*/
#include "vos.h"

#include "USBSlave.h"
#include "ioctl.h"
#include "USBSlaveFT232.h"
#include "GPIO.h"

#include "Vinculo_Test.h"

vos_tcb_t *tcbLoopback;

VOS_HANDLE hUSB1, hFT232, hGPIOA;

unsigned char readbuf[256];

void loopback(void);

void main(void)
{
    // GPIO context structure
    gpio_context_t gpioCtx;

    common_ioctl_cb_t ft232Attach;
    // GPIO IOCTL request block
    gpio_ioctl_cb_t gpio_iocb;

    vos_init(VOS_QUANTUM,VOS_TICK_INTERVAL,NUMBER_OF_DEVICES);
    vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);

    // Configure IOMUX
    // GPIO Port A 0 to pin 39 as Output.
    vos_iomux_define_output(39, IOMUX_OUT_GPIO_PORT_A_0); // LED1#
    // GPIO Port A 1 to pin 40 as Output.
    vos_iomux_define_output(40, IOMUX_OUT_GPIO_PORT_A_1); // LED2#
    // GPIO Port A 2 to pin 41 as Output.
    vos_iomux_define_output(41, IOMUX_OUT_GPIO_PORT_A_2); // PWREN#

    // initialise device drivers
    gpioCtx.port_identifier = GPIO_PORT_A;
    gpio_init(GPIO_A,&gpioCtx);

    usbslave_init(USBSLAVE_PORT_A, USBSA);
    usbslaveft232_init(USBSFT232);

    // open USB Slave port
    hUSB1 = vos_dev_open(USBSA);
    // open FT232BM
    hFT232 = vos_dev_open(USBSFT232);
    // open GPIO port A
    hGPIOA = vos_dev_open(GPIO_A);

    gpio_iocb.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
    gpio_iocb.value = 0x03;    // set lower 3 bits as outputs
    vos_dev_ioctl(hGPIOA, &gpio_iocb);

    // attach FT232BM to USB Slave port A
    ft232Attach.ioctl_code = VOS_IOCTL_USBSLAVEFT232_ATTACH;
    ft232Attach.set.data = hUSB1;
    vos_dev_ioctl(hFT232,&ft232Attach);

    tcbLoopback = vos_create_thread(31,TCB_SIZE,loopback,0);

    vos_start_scheduler();

main_loop:

    goto main_loop;
}

void loopback(void)
{
    common_ioctl_cb_t iocb;
    unsigned char b;

    b=0x03; // LED1#-High; LED2#-High; PWREN#-Low
    vos_dev_write(hGPIOA,&b,1,NULL);

    while(1)
    {
        // Check if there is any data to read
        iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
        iocb.get.queue_stat = 0;
        vos_dev_ioctl(hFT232,&iocb);
        if (iocb.get.queue_stat == 0)
        {
            // Blink LED2#
            b=0x01; // LED1#-High; LED2#-Low; PWREN#-Low
            vos_dev_write(hGPIOA,&b,1,NULL);
            vos_delay_msecs(20);
            b=0x03; // LED1#-High; LED2#-High; PWREN#-Low
            vos_dev_write(hGPIOA,&b,1,NULL);
            vos_delay_msecs(100);
            continue;
        }
        // Blink LED1#
        b=0x02; // LED1#-Low; LED2#-High; PWREN#-Low
        vos_dev_write(hGPIOA,&b,1,NULL);
        // Read data
        vos_dev_read(hFT232,readbuf,iocb.get.queue_stat,NULL);
        // Write data back
        vos_dev_write(hFT232,readbuf,iocb.get.queue_stat,NULL);

        vos_delay_msecs(20);
        b=0x03; // LED1#-High; LED2#-High; PWREN#-Low
        vos_dev_write(hGPIOA,&b,1,NULL);
    }

    return;
}


